@@ -476,6 +476,20 @@ nm_device_class_init (NMDeviceClass *device_class)
 						  NULL,
 						  G_PARAM_READABLE));
 
+	/**
+	 * NMDevice:ip-interface:
+	 *
+	 * The IP interface of the device which should be used for all IP-related
+	 * operations like addressing and routing.
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_IP_INTERFACE,
+		 g_param_spec_string (NM_DEVICE_IP_INTERFACE,
+						  "IP Interface",
+						  "IP Interface name",
+						  NULL,
+						  G_PARAM_READABLE));
+
 	/**
 	 * NMDevice:udi:
 	 *
